﻿using System;
using Microsoft.AnalysisServices;
using System.Text;
using AmoTabular;

namespace AmoSample {
    class Program {
        const string instanceName = "BISM\\Tabular";
        const string databaseName = "AmoTest";

        static void Main (string[] args) {
            //
            //  Connects to the instance
            //
            TabularProject tabularProject = new TabularProject ();
            tabularProject.Server.Connect (instanceName);
            tabularProject.CreateDatabase (databaseName);

            //
            //  Adds two connections, one with the DW, the other with OLTP
            //
            tabularProject.AddSqlServerConnection ("BISM", "AdventureWorksDW2012");
            //tabularProject.AddSqlServerConnection ("BISM", "AdventureWorks");

            //
            //  Now we can start adding tables to the database
            //
            tabularProject.AddTable ("AdventureWorksDW2012", "DimProduct", "SELECT ProductKey, ProductAlternateKey, ProductSubcategoryKey FROM DimProduct");
            tabularProject.AddTable ("AdventureWorksDW2012", "DimProductCategory", "SELECT ProductCategoryKey, EnglishProductCategoryName FROM DimProductCategory");
            tabularProject.AddTable ("AdventureWorksDW2012", "DimProductSubCategory", "SELECT ProductCategoryKey, ProductSubcategoryKey, EnglishProductSubcategoryName FROM DimProductSubCategory");
            //tabularProject.AddTable ("AdventureWorks", "Credit Cards", "SELECT * FROM Sales.CreditCard");

            tabularProject.Database.Update (UpdateOptions.ExpandFull, UpdateMode.UpdateOrCreate);


            //
            //  Adds some measures
            //
            //tabularProject.AddMeasure ("DimCustomer", "NumOfCustomers", "COUNTROWS (DimCustomer)", "###,###");
            tabularProject.AddMeasure ("DimProductCategory", "NumOfCategories", "COUNTROWS (DimProductCategory)", "###,###");

            //
            //  Adds a relationship between subcategories and categories
            //
            tabularProject.AddRelationship2 ("DimProductSubCategory", "ProductCategoryKey", "DimProductCategory", "ProductCategoryKey", true);
            tabularProject.AddRelationship2 ("DimProduct", "ProductSubcategoryKey", "DimProductSubcategory", "ProductSubcategoryKey", true);

            //
            //
            //  Let us add a calculated column to the model and check if relationships work
            //
            tabularProject.AddCalculatedColumn ("DimProduct", "SubcategoryName", "CALCULATE(VALUES (DimProductSubCategory[EnglishProductSubcategoryName]))");
            tabularProject.AddCalculatedColumn ("DimProduct", "CategoryName", "CALCULATE(VALUES (DimProductCategory[EnglishProductCategoryName]))");
            tabularProject.AddCalculatedColumn ("DimProduct", "CountOfCategory", "CALCULATE (COUNTROWS(VALUES (DimProductCategory[EnglishProductCategoryName])))");

            //
            //  Updates the database and process it
            //
            tabularProject.Database.Update (UpdateOptions.ExpandFull, UpdateMode.UpdateOrCreate);
            tabularProject.Database.Process (ProcessType.ProcessFull);

            tabularProject.Server.Disconnect ();
        }
    }
}
